#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import subprocess
import os
import re
from datetime import datetime
try:
    from crypto_utils import encrypt_string, decrypt_string
    CRYPTO_AVAILABLE = True
except ImportError:
    CRYPTO_AVAILABLE = False

ENCRYPTED_DOWNLOAD_COMMANDS = {
    'base_cmd': encrypt_string('yt-dlp.exe') if CRYPTO_AVAILABLE else 'eXQtZGxwLmV4ZQ==',
    'format_param': encrypt_string('--format') if CRYPTO_AVAILABLE else 'LS1mb3JtYXQ=',
    'output_param': encrypt_string('--output') if CRYPTO_AVAILABLE else 'LS1vdXRwdXQ=',
    'no_warnings': encrypt_string('--no-warnings') if CRYPTO_AVAILABLE else 'LS1uby13YXJuaW5ncw==',
    'newline': encrypt_string('--newline') if CRYPTO_AVAILABLE else 'LS1uZXdsaW5l',
    'concurrent': encrypt_string('--concurrent-fragments') if CRYPTO_AVAILABLE else 'LS1jb25jdXJyZW50LWZyYWdtZW50cw==',
    'retries': encrypt_string('--retries') if CRYPTO_AVAILABLE else 'LS1yZXRyaWVz',
    'fragment_retries': encrypt_string('--fragment-retries') if CRYPTO_AVAILABLE else 'LS1mcmFnbWVudC1yZXRyaWVz',
    'ffmpeg_location': encrypt_string('--ffmpeg-location') if CRYPTO_AVAILABLE else 'LS1mZm1wZWctbG9jYXRpb24=',
    'write_thumbnail': encrypt_string('--write-thumbnail') if CRYPTO_AVAILABLE else 'LS13cml0ZS10aHVtYm5haWw='
}

def _decrypt_command(encrypted_cmd):
    if CRYPTO_AVAILABLE:
        return decrypt_string(encrypted_cmd)
    else:
        import base64
        try:
            return base64.b64decode(encrypted_cmd).decode()
        except:
            return encrypted_cmd

class EncryptedDownloader:
    
    def __init__(self):
        self.is_downloading = False
        
    def build_download_command(self, url, quality, format_ext, output_dir, concurrent_count=3, download_cover=False):
        base_cmd = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['base_cmd'])
        format_param = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['format_param'])
        output_param = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['output_param'])
        no_warnings = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['no_warnings'])
        newline = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['newline'])
        concurrent = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['concurrent'])
        retries = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['retries'])
        fragment_retries = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['fragment_retries'])
        ffmpeg_location = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['ffmpeg_location'])
        write_thumbnail = _decrypt_command(ENCRYPTED_DOWNLOAD_COMMANDS['write_thumbnail'])
        
        format_selector = self._get_encrypted_format_selector(quality, format_ext)
        
        cmd = [
            os.path.join('bin', base_cmd),
            format_param, format_selector,
            output_param, os.path.join(output_dir, '%(title)s.%(ext)s'),
            no_warnings,
            newline,
            concurrent, str(concurrent_count),
            retries, '5',
            fragment_retries, '3',
        ]
        
        ffmpeg_path = os.path.join('bin', 'ffmpeg.exe')
        if os.path.exists(ffmpeg_path):
            cmd.extend([ffmpeg_location, ffmpeg_path])
        
        if download_cover:
            cmd.append(write_thumbnail)
            
        cmd.append(url)
        return cmd
    
    def _get_encrypted_format_selector(self, quality, format_ext):
        if format_ext in ['mp3', 'm4a', 'wav']:
            if format_ext == 'mp3':
                return 'bestaudio[ext=m4a]/bestaudio/best'
            else:
                return f'bestaudio[ext={format_ext}]/bestaudio/best'
        
        if quality == 'best':
            return 'best[ext=mp4]/best[ext=webm]/best'
        elif quality == 'worst':
            return 'worst[ext=mp4]/worst[ext=webm]/worst'
        else:
            height = quality.replace('p', '')
            selectors = [
                f'best[height<={height}][ext=mp4]',
                f'best[height<={height}][ext=webm]',
                f'best[height<={height}]',
                f'bestvideo[height<={height}]+bestaudio/best[height<={height}]',
                'best'
            ]
            
            if format_ext == 'mp4':
                return '/'.join(selectors)
            else:
                format_selectors = [f'{s}[ext={format_ext}]' for s in selectors[:3]]
                format_selectors.extend(selectors)
                return '/'.join(format_selectors)
    
    def execute_download(self, cmd, log_callback=None, progress_callback=None):
        try:
            process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True,
                encoding='utf-8',
                errors='replace'
            )
            
            while True:
                if not self.is_downloading:
                    process.terminate()
                    return False
                    
                line = process.stdout.readline()
                if not line:
                    break
                    
                line = line.strip()
                if line:
                    if log_callback:
                        log_callback(line)
                    
                    if '[download]' in line and '%' in line:
                        try:
                            progress_match = re.search(r'(\d+(?:\.\d+)?)%', line)
                            if progress_match:
                                progress = float(progress_match.group(1))
                                if progress_callback:
                                    progress_callback(progress)
                        except:
                            pass
                            
            process.wait()
            
            return process.returncode == 0
                
        except Exception as e:
            if log_callback:
                log_callback(f"❌ 下载出错: {e}")
            return False
    
    def start_download(self):
        self.is_downloading = True
    
    def stop_download(self):
        self.is_downloading = False

_encrypted_downloader = EncryptedDownloader()

def get_encrypted_downloader():
    return _encrypted_downloader 