#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import tkinter as tk
from tkinter import ttk, filedialog, messagebox, scrolledtext
import threading
import os
import sys
import subprocess
import json
import re
from urllib.parse import urlparse, parse_qs
import requests
from pathlib import Path
import tempfile
import shutil
import config
try:
    from crypto_utils import get_author_info
    from download_core import get_encrypted_downloader
    CRYPTO_AVAILABLE = True
except ImportError:
    CRYPTO_AVAILABLE = False

class YouTubeDownloader:
    def __init__(self, root):
        self.root = root
        if CRYPTO_AVAILABLE:
            author_info = get_author_info()
            main_title = author_info.get('main_title', '[Protected]')
            basic_suffix = author_info.get('basic_suffix', '[Protected]')
            window_title = main_title + " " + basic_suffix
        else:
            window_title = "[Protected]"
        
        self.root.title(window_title)
        self.root.geometry(config.WINDOW_SIZE)
        self.root.resizable(True, True)
        self.root.minsize(*config.WINDOW_MIN_SIZE)
        
        try:
            self.root.iconbitmap("icon.ico")
        except:
            pass
        
        self.download_path = config.DEFAULT_DOWNLOAD_PATH
        self.is_downloading = False
        
        os.makedirs(self.download_path, exist_ok=True)
        
        self.setup_ui()
        self.check_dependencies()
    
    def setup_ui(self):
        main_frame = ttk.Frame(self.root, padding="10")
        main_frame.grid(row=0, column=0, sticky=(tk.W, tk.E, tk.N, tk.S))
        
        self.root.columnconfigure(0, weight=1)
        self.root.rowconfigure(0, weight=1)
        main_frame.columnconfigure(1, weight=1)
        
        ttk.Label(main_frame, text="YouTube视频链接:", font=config.FONTS["default"]).grid(row=0, column=0, sticky=tk.W, pady=(0, 5))
        self.url_var = tk.StringVar()
        self.url_entry = ttk.Entry(main_frame, textvariable=self.url_var, width=60, font=config.FONTS["default"])
        self.url_entry.grid(row=0, column=1, sticky=(tk.W, tk.E), pady=(0, 5))
        
        ttk.Button(main_frame, text="粘贴", command=self.paste_url).grid(row=0, column=2, padx=(5, 0), pady=(0, 5))
        
        ttk.Label(main_frame, text="视频质量:", font=config.FONTS["default"]).grid(row=1, column=0, sticky=tk.W, pady=(0, 5))
        self.quality_var = tk.StringVar(value=config.DEFAULT_QUALITY)
        quality_combo = ttk.Combobox(main_frame, textvariable=self.quality_var, state="readonly", font=config.FONTS["default"])
        quality_combo['values'] = config.QUALITY_OPTIONS
        quality_combo.grid(row=1, column=1, sticky=tk.W, pady=(0, 5))
        
        ttk.Label(main_frame, text="输出格式:", font=config.FONTS["default"]).grid(row=2, column=0, sticky=tk.W, pady=(0, 5))
        self.format_var = tk.StringVar(value=config.DEFAULT_FORMAT)
        format_combo = ttk.Combobox(main_frame, textvariable=self.format_var, state="readonly", font=config.FONTS["default"])
        format_combo['values'] = config.FORMAT_OPTIONS
        format_combo.grid(row=2, column=1, sticky=tk.W, pady=(0, 5))
        
        ttk.Label(main_frame, text="下载路径:", font=config.FONTS["default"]).grid(row=3, column=0, sticky=tk.W, pady=(0, 5))
        self.path_var = tk.StringVar(value=self.download_path)
        path_entry = ttk.Entry(main_frame, textvariable=self.path_var, width=50, font=config.FONTS["default"])
        path_entry.grid(row=3, column=1, sticky=(tk.W, tk.E), pady=(0, 5))
        ttk.Button(main_frame, text="浏览", command=self.browse_path).grid(row=3, column=2, padx=(5, 0), pady=(0, 5))
        
        button_frame = ttk.Frame(main_frame)
        button_frame.grid(row=4, column=0, columnspan=3, pady=10, sticky=(tk.W, tk.E))
        
        self.download_btn = ttk.Button(button_frame, text="下载视频", command=self.start_download)
        self.download_btn.pack(side=tk.LEFT, padx=(0, 10))
        
        self.info_btn = ttk.Button(button_frame, text="获取视频信息", command=self.get_video_info)
        self.info_btn.pack(side=tk.LEFT, padx=(0, 10))
        
        self.clear_btn = ttk.Button(button_frame, text="清空日志", command=self.clear_log)
        self.clear_btn.pack(side=tk.LEFT)
        
        self.progress_var = tk.DoubleVar()
        self.progress_bar = ttk.Progressbar(main_frame, variable=self.progress_var, maximum=100)
        self.progress_bar.grid(row=5, column=0, columnspan=3, sticky=(tk.W, tk.E), pady=(10, 5))
        
        self.status_var = tk.StringVar(value=config.STATUS_MESSAGES["ready"])
        self.status_label = ttk.Label(main_frame, textvariable=self.status_var, font=config.FONTS["default"])
        self.status_label.grid(row=6, column=0, columnspan=3, sticky=tk.W)
        
        ttk.Label(main_frame, text="下载日志:", font=config.FONTS["default"]).grid(row=7, column=0, sticky=tk.W, pady=(10, 5))
        self.log_text = scrolledtext.ScrolledText(main_frame, height=15, width=80, font=config.FONTS["log"])
        self.log_text.grid(row=8, column=0, columnspan=3, sticky=(tk.W, tk.E, tk.N, tk.S), pady=(0, 10))
        
        main_frame.rowconfigure(8, weight=1)
    
    def paste_url(self):
        try:
            clipboard_content = self.root.clipboard_get()
            if "youtube.com" in clipboard_content or "youtu.be" in clipboard_content:
                self.url_var.set(clipboard_content)
                self.log("已粘贴URL: " + clipboard_content)
            else:
                messagebox.showwarning("警告", "剪贴板内容不是有效的YouTube链接")
        except:
            messagebox.showerror("错误", "无法访问剪贴板")
    
    def browse_path(self):
        path = filedialog.askdirectory(initialdir=self.path_var.get())
        if path:
            self.path_var.set(path)
            self.download_path = path
    
    def log(self, message):
        self.log_text.insert(tk.END, f"{message}\n")
        self.log_text.see(tk.END)
        self.root.update_idletasks()
    
    def clear_log(self):
        self.log_text.delete(1.0, tk.END)
    
    def update_status(self, status):
        self.status_var.set(status)
        self.root.update_idletasks()
    
    def update_progress(self, progress):
        self.progress_var.set(progress)
        self.root.update_idletasks()
    
    def check_dependencies(self):
        self.log("检查依赖项...")
        self.update_status(config.STATUS_MESSAGES["checking_deps"])
        
        if not self.check_ytdlp():
            self.log("正在下载yt-dlp...")
            if self.download_ytdlp():
                self.log("yt-dlp下载完成")
            else:
                self.log("yt-dlp下载失败，请检查网络连接")
                messagebox.showerror("错误", "无法下载必要组件，请检查网络连接")
        else:
            self.log("yt-dlp已就绪")
        
        if not self.check_ffmpeg():
            self.log("正在下载ffmpeg...")
            if self.download_ffmpeg():
                self.log("ffmpeg下载完成")
            else:
                self.log("ffmpeg下载失败，某些功能可能受限")
        else:
            self.log("ffmpeg已就绪")
        
        self.log(config.SUCCESS_MESSAGES["dependencies_ready"])
        self.update_status(config.STATUS_MESSAGES["ready"])
    
    def check_ytdlp(self):
        return os.path.exists(config.YTDLP_PATH)
    
    def check_ffmpeg(self):
        return os.path.exists(config.FFMPEG_PATH)
    
    def download_ytdlp(self):
        try:
            response = requests.get(config.YTDLP_DOWNLOAD_URL, stream=True, timeout=config.DOWNLOAD_TIMEOUT)
            response.raise_for_status()
            
            with open(config.YTDLP_PATH, 'wb') as f:
                for chunk in response.iter_content(chunk_size=config.CHUNK_SIZE):
                    f.write(chunk)
            
            return True
        except Exception as e:
            self.log(f"下载yt-dlp失败: {str(e)}")
            return False
    
    def download_ffmpeg(self):
        try:
            self.log("ffmpeg下载功能需要手动实现")
            return False
        except Exception as e:
            self.log(f"下载ffmpeg失败: {str(e)}")
            return False
    
    def get_video_info(self):
        url = self.url_var.get().strip()
        if not url:
            messagebox.showwarning("警告", config.ERROR_MESSAGES["no_url"])
            return
        
        if not self.is_valid_youtube_url(url):
            messagebox.showwarning("警告", config.ERROR_MESSAGES["invalid_url"])
            return
        
        def info_thread():
            try:
                self.update_status(config.STATUS_MESSAGES["getting_info"])
                self.info_btn.config(state="disabled")
                
                if not os.path.exists(config.YTDLP_PATH):
                    self.log("错误: " + config.ERROR_MESSAGES["ytdlp_not_found"])
                    return
                
                cmd = [config.YTDLP_PATH, "--dump-json", url]
                result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8')
                
                if result.returncode == 0:
                    info = json.loads(result.stdout)
                    self.log(f"标题: {info.get('title', 'N/A')}")
                    self.log(f"时长: {info.get('duration', 'N/A')}秒")
                    self.log(f"上传者: {info.get('uploader', 'N/A')}")
                    self.log(f"观看次数: {info.get('view_count', 'N/A')}")
                    self.log(f"上传日期: {info.get('upload_date', 'N/A')}")
                    self.log(config.SUCCESS_MESSAGES["info_retrieved"])
                else:
                    self.log(f"获取视频信息失败: {result.stderr}")
                
            except Exception as e:
                self.log(f"获取视频信息时出错: {str(e)}")
            finally:
                self.update_status(config.STATUS_MESSAGES["ready"])
                self.info_btn.config(state="normal")
        
        threading.Thread(target=info_thread, daemon=True).start()
    
    def is_valid_youtube_url(self, url):
        for pattern in config.YOUTUBE_URL_PATTERNS:
            if re.match(pattern, url):
                return True
        return False
    
    def start_download(self):
        url = self.url_var.get().strip()
        if not url:
            messagebox.showwarning("警告", config.ERROR_MESSAGES["no_url"])
            return
        
        if not self.is_valid_youtube_url(url):
            messagebox.showwarning("警告", config.ERROR_MESSAGES["invalid_url"])
            return
        
        if self.is_downloading:
            messagebox.showinfo("提示", "正在下载中，请等待当前下载完成")
            return
        
        threading.Thread(target=self.download_video, args=(url,), daemon=True).start()
    
    def download_video(self, url):
        try:
            self.is_downloading = True
            self.download_btn.config(state="disabled")
            self.update_status(config.STATUS_MESSAGES["downloading"])
            self.update_progress(0)
            
            if not os.path.exists(config.YTDLP_PATH):
                self.log("错误: " + config.ERROR_MESSAGES["ytdlp_not_found"])
                return
            
            output_path = os.path.join(self.path_var.get(), "%(title)s.%(ext)s")
            quality = self.quality_var.get()
            format_type = self.format_var.get()
            
            cmd = [config.YTDLP_PATH]
            
            cmd.extend(["-o", output_path])
            
            if format_type in config.AUDIO_FORMATS:
                cmd.extend(["-x", "--audio-format", format_type])
            else:
                if quality == "best":
                    cmd.extend(["-f", f"best[ext={format_type}]/best"])
                elif quality == "worst":
                    cmd.extend(["-f", f"worst[ext={format_type}]/worst"])
                else:
                    height = quality.replace("p", "")
                    cmd.extend(["-f", f"best[height<={height}][ext={format_type}]/best[height<={height}]/best"])
            
            cmd.extend(["--newline"])
            
            cmd.append(url)
            
            self.log(f"开始下载: {url}")
            
            process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True,
                encoding='utf-8',
                universal_newlines=True
            )
            
            for line in process.stdout:
                line = line.strip()
                if line:
                    self.log(line)
                    
                    if "[download]" in line and "%" in line:
                        try:
                            percent_match = re.search(r'(\d+(?:\.\d+)?)%', line)
                            if percent_match:
                                percent = float(percent_match.group(1))
                                self.update_progress(percent)
                        except:
                            pass
            
            process.wait()
            
            if process.returncode == 0:
                self.log(config.SUCCESS_MESSAGES["download_complete"])
                self.update_status(config.STATUS_MESSAGES["download_complete"])
                self.update_progress(100)
                messagebox.showinfo("成功", config.SUCCESS_MESSAGES["download_complete"])
            else:
                self.log(config.ERROR_MESSAGES["download_failed"])
                self.update_status(config.STATUS_MESSAGES["download_failed"])
                messagebox.showerror("错误", config.ERROR_MESSAGES["download_failed"])
            
        except Exception as e:
            self.log(f"下载时出错: {str(e)}")
            self.update_status(config.STATUS_MESSAGES["download_failed"])
            messagebox.showerror("错误", f"下载时出错: {str(e)}")
        finally:
            self.is_downloading = False
            self.download_btn.config(state="normal")
            self.update_progress(0)

def main():
    root = tk.Tk()
    app = YouTubeDownloader(root)
    
    def on_closing():
        if app.is_downloading:
            if messagebox.askokcancel("退出", "正在下载中，确定要退出吗？"):
                root.destroy()
        else:
            root.destroy()
    
    root.protocol("WM_DELETE_WINDOW", on_closing)
    
    root.mainloop()

if __name__ == "__main__":
    main() 