#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import re
import json
import requests
import subprocess
from pathlib import Path
from urllib.parse import urlparse, parse_qs
import tempfile
import shutil

def format_size(size_bytes):
    if size_bytes == 0:
        return "0B"
    
    size_names = ["B", "KB", "MB", "GB", "TB"]
    i = 0
    while size_bytes >= 1024 and i < len(size_names) - 1:
        size_bytes /= 1024.0
        i += 1
    
    return f"{size_bytes:.1f}{size_names[i]}"

def format_duration(seconds):
    if not seconds:
        return "未知"
    
    try:
        seconds = int(seconds)
        hours = seconds // 3600
        minutes = (seconds % 3600) // 60
        seconds = seconds % 60
        
        if hours > 0:
            return f"{hours:02d}:{minutes:02d}:{seconds:02d}"
        else:
            return f"{minutes:02d}:{seconds:02d}"
    except:
        return "未知"

def sanitize_filename(filename):
    illegal_chars = r'[<>:"/\\|?*]'
    filename = re.sub(illegal_chars, '_', filename)
    filename = filename.strip(' .')
    if len(filename) > 200:
        filename = filename[:200]
    return filename

def extract_video_id(url):
    patterns = [
        r'(?:v=|\/)([0-9A-Za-z_-]{11}).*',
        r'(?:embed\/)([0-9A-Za-z_-]{11})',
        r'(?:youtu\.be\/)([0-9A-Za-z_-]{11})'
    ]
    
    for pattern in patterns:
        match = re.search(pattern, url)
        if match:
            return match.group(1)
    
    return None

def get_video_thumbnail_url(video_id):
    if not video_id:
        return None
    
    thumbnail_urls = [
        f"https://img.youtube.com/vi/{video_id}/maxresdefault.jpg",
        f"https://img.youtube.com/vi/{video_id}/hqdefault.jpg",
        f"https://img.youtube.com/vi/{video_id}/mqdefault.jpg",
        f"https://img.youtube.com/vi/{video_id}/default.jpg"
    ]
    
    return thumbnail_urls[0]

def check_internet_connection():
    try:
        response = requests.get("https://www.google.com", timeout=5)
        return response.status_code == 200
    except:
        return False

def get_available_formats(url, ytdlp_path):
    try:
        cmd = [ytdlp_path, "--list-formats", url]
        result = subprocess.run(cmd, capture_output=True, text=True, encoding='utf-8')
        
        if result.returncode == 0:
            return result.stdout
        else:
            return None
    except Exception:
        return None

def download_file(url, filepath, chunk_size=8192, progress_callback=None):
    try:
        response = requests.get(url, stream=True)
        response.raise_for_status()
        
        total_size = int(response.headers.get('content-length', 0))
        downloaded = 0
        
        with open(filepath, 'wb') as f:
            for chunk in response.iter_content(chunk_size=chunk_size):
                if chunk:
                    f.write(chunk)
                    downloaded += len(chunk)
                    
                    if progress_callback and total_size > 0:
                        progress = (downloaded / total_size) * 100
                        progress_callback(progress)
        
        return True
    except Exception:
        return False

def create_desktop_shortcut(target_path, shortcut_name):
    try:
        import winshell
        from win32com.client import Dispatch
        
        desktop = winshell.desktop()
        shortcut_path = os.path.join(desktop, f"{shortcut_name}.lnk")
        
        shell = Dispatch('WScript.Shell')
        shortcut = shell.CreateShortCut(shortcut_path)
        shortcut.Targetpath = target_path
        shortcut.WorkingDirectory = os.path.dirname(target_path)
        shortcut.IconLocation = target_path
        shortcut.save()
        
        return True
    except:
        return False

def get_system_info():
    import platform
    
    info = {
        'system': platform.system(),
        'release': platform.release(),
        'version': platform.version(),
        'machine': platform.machine(),
        'processor': platform.processor(),
        'python_version': platform.python_version()
    }
    
    return info

def validate_download_path(path):
    try:
        if not os.path.exists(path):
            os.makedirs(path, exist_ok=True)
        
        test_file = os.path.join(path, 'test_write.tmp')
        with open(test_file, 'w') as f:
            f.write('test')
        os.remove(test_file)
        
        return True
    except:
        return False

def get_disk_space(path):
    try:
        if os.name == 'nt':
            import shutil
            total, used, free = shutil.disk_usage(path)
            return {
                'total': total,
                'used': used,
                'free': free,
                'free_formatted': format_size(free)
            }
        else:
            statvfs = os.statvfs(path)
            free = statvfs.f_frsize * statvfs.f_bavail
            total = statvfs.f_frsize * statvfs.f_blocks
            used = total - free
            return {
                'total': total,
                'used': used,
                'free': free,
                'free_formatted': format_size(free)
            }
    except:
        return None

def parse_progress_line(line):
    progress_info = {}
    
    percent_match = re.search(r'(\d+(?:\.\d+)?)%', line)
    if percent_match:
        progress_info['percent'] = float(percent_match.group(1))
    
    speed_match = re.search(r'(\d+(?:\.\d+)?[KMGT]?iB/s)', line)
    if speed_match:
        progress_info['speed'] = speed_match.group(1)
    
    eta_match = re.search(r'ETA (\d+:\d+)', line)
    if eta_match:
        progress_info['eta'] = eta_match.group(1)
    
    size_match = re.search(r'(\d+(?:\.\d+)?[KMGT]?iB)', line)
    if size_match:
        progress_info['size'] = size_match.group(1)
    
    return progress_info

def cleanup_temp_files(temp_dir):
    try:
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)
        return True
    except:
        return False

def get_config_value(config_file, key, default=None):
    try:
        with open(config_file, 'r', encoding='utf-8') as f:
            config = json.load(f)
        return config.get(key, default)
    except:
        return default

def save_config_value(config_file, key, value):
    try:
        config = {}
        if os.path.exists(config_file):
            with open(config_file, 'r', encoding='utf-8') as f:
                config = json.load(f)
        
        config[key] = value
        
        with open(config_file, 'w', encoding='utf-8') as f:
            json.dump(config, f, indent=2, ensure_ascii=False)
        
        return True
    except:
        return False 