#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import tkinter as tk
from tkinter import ttk, filedialog, messagebox, scrolledtext
import threading
import subprocess
import os
import sys
import re
import json
import time
from datetime import datetime
import requests
from urllib.parse import urlparse, parse_qs
try:
    from crypto_utils import get_author_info, encrypt_string, decrypt_string
    from download_core import get_encrypted_downloader
    CRYPTO_AVAILABLE = True
except ImportError:
    CRYPTO_AVAILABLE = False
    ENCRYPTED_BACKUP = {
        'author_name': 'gAAAAABnYXV0aG9yX25hbWVfZW5jcnlwdGVkX2RhdGE=',
        'author_website': 'gAAAAABnYXV0aG9yX3dlYnNpdGVfZW5jcnlwdGVkX2RhdGE=',
        'version': 'gAAAAABnYXV0aG9yX3ZlcnNpb25fZW5jcnlwdGVkX2RhdGE=',
        'app_title': 'gAAAAABnYXV0aG9yX2FwcF90aXRsZV9lbmNyeXB0ZWRfZGF0YQ=='
    }

class ModernColors:
    PRIMARY = "#2563eb"
    PRIMARY_DARK = "#1d4ed8"
    SECONDARY = "#64748b"
    SUCCESS = "#10b981"
    WARNING = "#f59e0b"
    ERROR = "#ef4444"
    BACKGROUND = "#f8fafc"
    SURFACE = "#ffffff"
    TEXT = "#1e293b"
    TEXT_LIGHT = "#64748b"
    BORDER = "#e2e8f0"

class YouTubeDownloader:
    def __init__(self):
        self.root = tk.Tk()
        self.setup_window()
        self.setup_variables()
        
        self.ensure_directories()
        
        self.check_dependencies()
        
        self.create_widgets()
        
        self.bind_events()
        
    def setup_window(self):
        try:
            if CRYPTO_AVAILABLE:
                author_info = get_author_info()
                window_title = author_info.get('main_title', '[Protected]')
            else:
                window_title = '[Protected]'
        except Exception as e:
            window_title = '[Protected]'
        
        self.root.title(window_title)
        self.root.geometry("1000x750")
        self.root.configure(bg=ModernColors.BACKGROUND)
        self.root.resizable(True, True)
        
        self.root.minsize(800, 600)
        
        self.center_window()
        
        try:
            self.root.iconbitmap('icon.ico')
        except:
            pass
            
    def center_window(self):
        self.root.update_idletasks()
        width = self.root.winfo_width()
        height = self.root.winfo_height()
        x = (self.root.winfo_screenwidth() // 2) - (width // 2)
        y = (self.root.winfo_screenheight() // 2) - (height // 2)
        self.root.geometry(f'{width}x{height}+{x}+{y}')
        
    def setup_variables(self):
        self.url_var = tk.StringVar()
        self.quality_var = tk.StringVar(value="720p")
        self.format_var = tk.StringVar(value="mp4")
        self.output_path_var = tk.StringVar(value=os.path.join(os.getcwd(), "downloads"))
        self.progress_var = tk.DoubleVar()
        self.status_var = tk.StringVar(value="就绪")
        self.download_cover_var = tk.BooleanVar(value=False)
        self.concurrent_var = tk.IntVar(value=3)
        
        self.download_queue = []
        self.is_downloading = False
        
    def ensure_directories(self):
        directories = ['bin', 'downloads']
        for directory in directories:
            if not os.path.exists(directory):
                os.makedirs(directory)
                
    def check_dependencies(self):
        yt_dlp_path = os.path.join('bin', 'yt-dlp.exe')
        if not os.path.exists(yt_dlp_path):
            self.log_message("正在下载yt-dlp...")
            try:
                response = requests.get(
                    'https://github.com/yt-dlp/yt-dlp/releases/latest/download/yt-dlp.exe',
                    timeout=30
                )
                with open(yt_dlp_path, 'wb') as f:
                    f.write(response.content)
                self.log_message("yt-dlp下载完成")
            except Exception as e:
                self.log_message(f"yt-dlp下载失败: {e}")
                
        ffmpeg_path = os.path.join('bin', 'ffmpeg.exe')
        if not os.path.exists(ffmpeg_path):
            self.log_message("警告: 未找到ffmpeg，某些功能可能受限")
            
    def create_widgets(self):
        main_container = tk.Frame(self.root, bg=ModernColors.BACKGROUND)
        main_container.pack(fill=tk.BOTH, expand=True, padx=20, pady=20)
        
        self.create_header(main_container)
        
        content_frame = tk.Frame(main_container, bg=ModernColors.BACKGROUND)
        content_frame.pack(fill=tk.BOTH, expand=True, pady=(20, 0))
        
        left_panel = tk.Frame(content_frame, bg=ModernColors.BACKGROUND)
        left_panel.pack(side=tk.LEFT, fill=tk.BOTH, expand=True, padx=(0, 10))
        
        right_panel = tk.Frame(content_frame, bg=ModernColors.BACKGROUND, width=300)
        right_panel.pack(side=tk.RIGHT, fill=tk.Y, padx=(10, 0))
        right_panel.pack_propagate(False)
        
        self.create_url_section(left_panel)
        self.create_settings_section(left_panel)
        self.create_control_section(left_panel)
        self.create_progress_section(left_panel)
        self.create_log_section(left_panel)
        
        self.create_queue_section(right_panel)
        self.create_info_section(right_panel)
        
    def create_header(self, parent):
        header_frame = tk.Frame(parent, bg=ModernColors.SURFACE, relief='solid', bd=1)
        header_frame.pack(fill=tk.X, pady=(0, 20))
        
        header_content = tk.Frame(header_frame, bg=ModernColors.SURFACE)
        header_content.pack(fill=tk.X, padx=20, pady=15)
        
        try:
            if CRYPTO_AVAILABLE:
                author_info = get_author_info()
                title_text = author_info.get('title_with_icon', '[Protected]')
            else:
                title_text = '[Protected]'
        except:
            title_text = '[Protected]'
            
        title_label = tk.Label(header_content, text=title_text, 
                              font=("Microsoft YaHei UI", 16, 'bold'),
                              fg=ModernColors.TEXT, bg=ModernColors.SURFACE)
        title_label.pack(side=tk.LEFT)
        
        author_frame = tk.Frame(header_content, bg=ModernColors.SURFACE)
        author_frame.pack(side=tk.RIGHT)
        
        if CRYPTO_AVAILABLE:
            author_info = get_author_info()
            author_name = author_info.get('author_name', '[Protected]')
            version = author_info.get('version_number', '[Protected]')
        else:
            author_name = '[Protected]'
            version = '[Protected]'
        
        version_label = tk.Label(author_frame, text=f"{version}", 
                               font=("Microsoft YaHei UI", 10),
                               fg=ModernColors.TEXT_LIGHT, bg=ModernColors.SURFACE)
        version_label.pack(side=tk.RIGHT)
        
        author_label = tk.Label(author_frame, text=f"作者: {author_name} | ", 
                               font=("Microsoft YaHei UI", 10),
                               fg=ModernColors.TEXT_LIGHT, bg=ModernColors.SURFACE)
        author_label.pack(side=tk.RIGHT)
        
    def create_url_section(self, parent):
        url_frame = tk.LabelFrame(parent, text="📺 视频链接", 
                                 font=("Microsoft YaHei UI", 10, 'bold'),
                                 fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                 relief='solid', bd=1)
        url_frame.pack(fill=tk.X, pady=(0, 15), padx=5, ipady=10)
        
        url_container = tk.Frame(url_frame, bg=ModernColors.SURFACE)
        url_container.pack(fill=tk.X, padx=15, pady=(10, 15))
        
        self.url_entry = tk.Entry(url_container, textvariable=self.url_var, 
                                 font=("Microsoft YaHei UI", 11),
                                 relief='solid', bd=1)
        self.url_entry.pack(side=tk.LEFT, fill=tk.X, expand=True, padx=(0, 10), ipady=5)
        
        button_container = tk.Frame(url_frame, bg=ModernColors.SURFACE)
        button_container.pack(fill=tk.X, padx=15)
        
        paste_btn = tk.Button(button_container, text="📋 粘贴", command=self.paste_url,
                             font=("Microsoft YaHei UI", 9), relief='solid', bd=1,
                             bg=ModernColors.PRIMARY, fg='white', cursor='hand2')
        paste_btn.pack(side=tk.LEFT, ipadx=10, ipady=3)
        
    def create_settings_section(self, parent):
        settings_frame = tk.LabelFrame(parent, text="⚙️ 下载设置", 
                                      font=("Microsoft YaHei UI", 10, 'bold'),
                                      fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                      relief='solid', bd=1)
        settings_frame.pack(fill=tk.X, pady=(0, 15), padx=5, ipady=10)
        
        row1 = tk.Frame(settings_frame, bg=ModernColors.SURFACE)
        row1.pack(fill=tk.X, padx=15, pady=(10, 10))
        
        quality_frame = tk.Frame(row1, bg=ModernColors.SURFACE)
        quality_frame.pack(side=tk.LEFT, fill=tk.X, expand=True, padx=(0, 15))
        
        tk.Label(quality_frame, text="视频质量:", 
                font=("Microsoft YaHei UI", 9),
                fg=ModernColors.TEXT, bg=ModernColors.SURFACE).pack(anchor=tk.W)
        
        quality_combo = ttk.Combobox(quality_frame, textvariable=self.quality_var,
                                   values=["2160p", "1440p", "1080p", "720p", "480p", "360p", "240p", "144p", "best", "worst"],
                                   state="readonly", width=12, font=("Microsoft YaHei UI", 9))
        quality_combo.pack(fill=tk.X, pady=(5, 0))
        
        format_frame = tk.Frame(row1, bg=ModernColors.SURFACE)
        format_frame.pack(side=tk.LEFT, fill=tk.X, expand=True, padx=(0, 15))
        
        tk.Label(format_frame, text="输出格式:", 
                font=("Microsoft YaHei UI", 9),
                fg=ModernColors.TEXT, bg=ModernColors.SURFACE).pack(anchor=tk.W)
        
        format_combo = ttk.Combobox(format_frame, textvariable=self.format_var,
                                  values=["mp4", "webm", "mkv", "mp3", "m4a", "wav"],
                                  state="readonly", width=12, font=("Microsoft YaHei UI", 9))
        format_combo.pack(fill=tk.X, pady=(5, 0))
        
        concurrent_frame = tk.Frame(row1, bg=ModernColors.SURFACE)
        concurrent_frame.pack(side=tk.LEFT, fill=tk.X, expand=True)
        
        tk.Label(concurrent_frame, text="并发数:", 
                font=("Microsoft YaHei UI", 9),
                fg=ModernColors.TEXT, bg=ModernColors.SURFACE).pack(anchor=tk.W)
        
        concurrent_spin = tk.Spinbox(concurrent_frame, from_=1, to=6, textvariable=self.concurrent_var, 
                                   width=8, font=("Microsoft YaHei UI", 9), relief='solid', bd=1)
        concurrent_spin.pack(fill=tk.X, pady=(5, 0))
        
        row2 = tk.Frame(settings_frame, bg=ModernColors.SURFACE)
        row2.pack(fill=tk.X, padx=15, pady=(0, 10))
        
        tk.Label(row2, text="保存路径:", 
                font=("Microsoft YaHei UI", 9),
                fg=ModernColors.TEXT, bg=ModernColors.SURFACE).pack(anchor=tk.W)
        
        path_container = tk.Frame(row2, bg=ModernColors.SURFACE)
        path_container.pack(fill=tk.X, pady=(5, 0))
        
        self.path_entry = tk.Entry(path_container, textvariable=self.output_path_var, 
                                  font=("Microsoft YaHei UI", 9), relief='solid', bd=1)
        self.path_entry.pack(side=tk.LEFT, fill=tk.X, expand=True, padx=(0, 10), ipady=3)
        
        browse_btn = tk.Button(path_container, text="📁 浏览", command=self.browse_path,
                              font=("Microsoft YaHei UI", 8), relief='solid', bd=1,
                              bg=ModernColors.SECONDARY, fg='white', cursor='hand2')
        browse_btn.pack(side=tk.LEFT, padx=(0, 5), ipadx=8, ipady=2)
        
        open_btn = tk.Button(path_container, text="🔗 打开", command=self.open_folder,
                            font=("Microsoft YaHei UI", 8), relief='solid', bd=1,
                            bg=ModernColors.SECONDARY, fg='white', cursor='hand2')
        open_btn.pack(side=tk.LEFT, ipadx=8, ipady=2)
        
        row3 = tk.Frame(settings_frame, bg=ModernColors.SURFACE)
        row3.pack(fill=tk.X, padx=15, pady=(0, 5))
        
        cover_check = tk.Checkbutton(row3, text="📷 下载封面", variable=self.download_cover_var,
                                   font=("Microsoft YaHei UI", 9),
                                   fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                   selectcolor=ModernColors.SURFACE)
        cover_check.pack(side=tk.LEFT)
        
    def create_control_section(self, parent):
        control_frame = tk.Frame(parent, bg=ModernColors.BACKGROUND)
        control_frame.pack(fill=tk.X, pady=(0, 15))
        
        self.download_btn = tk.Button(control_frame, text="🚀 开始下载", 
                                     command=self.start_download,
                                     font=("Microsoft YaHei UI", 10, 'bold'),
                                     bg=ModernColors.PRIMARY, fg='white',
                                     relief='solid', bd=1, cursor='hand2')
        self.download_btn.pack(side=tk.LEFT, padx=(0, 10), ipadx=15, ipady=8)
        
        add_btn = tk.Button(control_frame, text="➕ 添加到队列", command=self.add_to_queue,
                           font=("Microsoft YaHei UI", 9), relief='solid', bd=1,
                           bg=ModernColors.SUCCESS, fg='white', cursor='hand2')
        add_btn.pack(side=tk.LEFT, padx=(0, 10), ipadx=10, ipady=5)
        
        clear_btn = tk.Button(control_frame, text="🗑️ 清空队列", command=self.clear_queue,
                             font=("Microsoft YaHei UI", 9), relief='solid', bd=1,
                             bg=ModernColors.WARNING, fg='white', cursor='hand2')
        clear_btn.pack(side=tk.LEFT, padx=(0, 10), ipadx=10, ipady=5)
        
        stop_btn = tk.Button(control_frame, text="⏹️ 停止下载", command=self.stop_download,
                            font=("Microsoft YaHei UI", 9), relief='solid', bd=1,
                            bg=ModernColors.ERROR, fg='white', cursor='hand2')
        stop_btn.pack(side=tk.LEFT, ipadx=10, ipady=5)
        
    def create_progress_section(self, parent):
        progress_frame = tk.LabelFrame(parent, text="📊 下载进度", 
                                      font=("Microsoft YaHei UI", 10, 'bold'),
                                      fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                      relief='solid', bd=1)
        progress_frame.pack(fill=tk.X, pady=(0, 15), padx=5, ipady=10)
        
        self.progress_bar = ttk.Progressbar(progress_frame, variable=self.progress_var, maximum=100)
        self.progress_bar.pack(fill=tk.X, padx=15, pady=(10, 10))
        
        self.status_label = tk.Label(progress_frame, textvariable=self.status_var,
                                    font=("Microsoft YaHei UI", 9),
                                    fg=ModernColors.TEXT_LIGHT, bg=ModernColors.SURFACE)
        self.status_label.pack(anchor=tk.W, padx=15, pady=(0, 5))
        
    def create_log_section(self, parent):
        log_frame = tk.LabelFrame(parent, text="📝 下载日志", 
                                 font=("Microsoft YaHei UI", 10, 'bold'),
                                 fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                 relief='solid', bd=1)
        log_frame.pack(fill=tk.BOTH, expand=True, padx=5, ipady=5)
        
        self.log_text = scrolledtext.ScrolledText(
            log_frame, 
            height=12, 
            wrap=tk.WORD,
            font=("Consolas", 9),
            bg=ModernColors.SURFACE,
            fg=ModernColors.TEXT,
            relief='solid',
            bd=1
        )
        self.log_text.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)
        
    def create_queue_section(self, parent):
        queue_frame = tk.LabelFrame(parent, text="📋 下载队列", 
                                   font=("Microsoft YaHei UI", 10, 'bold'),
                                   fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                   relief='solid', bd=1)
        queue_frame.pack(fill=tk.BOTH, expand=True, pady=(0, 15), padx=5, ipady=5)
        
        queue_btn_frame = tk.Frame(queue_frame, bg=ModernColors.SURFACE)
        queue_btn_frame.pack(fill=tk.X, padx=10, pady=(5, 0))
        
        add_queue_btn = tk.Button(queue_btn_frame, text="➕ 添加到队列",
                                 command=self.add_to_queue,
                                 font=("Microsoft YaHei UI", 9),
                                 bg=ModernColors.PRIMARY, fg="white",
                                 relief=tk.FLAT, padx=15, pady=5)
        add_queue_btn.pack(side=tk.LEFT, padx=(0, 5))
        
        redownload_btn = tk.Button(queue_btn_frame, text="🔄 重新下载",
                                  command=self.redownload_selected,
                                  font=("Microsoft YaHei UI", 9),
                                  bg=ModernColors.WARNING, fg="white",
                                  relief=tk.FLAT, padx=15, pady=5)
        redownload_btn.pack(side=tk.LEFT, padx=(0, 5))
        
        clear_queue_btn = tk.Button(queue_btn_frame, text="🗑️ 清空队列",
                                   command=self.clear_queue,
                                   font=("Microsoft YaHei UI", 9),
                                   bg=ModernColors.ERROR, fg="white",
                                   relief=tk.FLAT, padx=15, pady=5)
        clear_queue_btn.pack(side=tk.LEFT)
        
        self.queue_tree = ttk.Treeview(queue_frame, columns=('quality', 'format', 'status'), 
                                      show='headings', height=6)
        self.queue_tree.heading('quality', text='质量')
        self.queue_tree.heading('format', text='格式')
        self.queue_tree.heading('status', text='状态')
        
        self.queue_tree.column('quality', width=60)
        self.queue_tree.column('format', width=60)
        self.queue_tree.column('status', width=80)
        
        self.queue_tree.pack(fill=tk.BOTH, expand=True, padx=10, pady=10)
        
    def create_info_section(self, parent):
        info_frame = tk.LabelFrame(parent, text="ℹ️ 软件信息", 
                                  font=("Microsoft YaHei UI", 10, 'bold'),
                                  fg=ModernColors.TEXT, bg=ModernColors.SURFACE,
                                  relief='solid', bd=1)
        info_frame.pack(fill=tk.X, padx=5, ipady=10)
        
        info_text = """🎯 功能特点:
• 支持多种视频质量
• 支持音频/视频格式
• 批量下载队列
• 实时进度显示
• 自动音视频合并

🔧 支持格式:
• 视频: MP4, WebM, MKV
• 音频: MP3, M4A, WAV
• 质量: 144p - 2160p

💡 使用提示:
• 复制并粘贴YouTube链接
• 选择质量和格式
• 点击"添加到队列"按钮
• 再点击"开始下载"按钮"""
        
        info_label = tk.Label(info_frame, text=info_text, 
                             font=("Microsoft YaHei UI", 8),
                             fg=ModernColors.TEXT_LIGHT, bg=ModernColors.SURFACE,
                             justify=tk.LEFT, anchor=tk.NW)
        info_label.pack(fill=tk.BOTH, expand=True, padx=15, pady=(10, 5))
        
        try:
            if CRYPTO_AVAILABLE:
                author_info = get_author_info()
                author_name = author_info.get('author_name', '[Protected]')
                author_website = author_info.get('author_website', '[Protected]')
            else:
                author_name = '[Protected]'
                author_website = '[Protected]'
        except:
            author_name = '[Protected]'
            author_website = '[Protected]'
        
        author_info_frame = tk.Frame(info_frame, bg=ModernColors.SURFACE)
        author_info_frame.pack(fill=tk.X, padx=15, pady=(0, 10))
        
        author_text = f"👤 作者: {author_name}"
        author_label = tk.Label(author_info_frame, text=author_text,
                               font=("Microsoft YaHei UI", 8),
                               fg=ModernColors.TEXT_LIGHT, bg=ModernColors.SURFACE,
                               anchor=tk.W)
        author_label.pack(fill=tk.X)
        
        website_text = f"🌐 作者主页: {author_website}"
        website_label = tk.Label(author_info_frame, text=website_text,
                                font=("Microsoft YaHei UI", 8),
                                fg=ModernColors.PRIMARY, bg=ModernColors.SURFACE,
                                anchor=tk.W, cursor="hand2")
        website_label.pack(fill=tk.X)
        
        def open_website(event):
            try:
                if author_website != '[Protected]':
                    import webbrowser
                    webbrowser.open(author_website)
            except:
                pass
        
        website_label.bind("<Button-1>", open_website)
        
    def bind_events(self):
        self.root.bind('<Control-v>', self.paste_url)
        
    def paste_url(self, event=None):
        try:
            clipboard_content = self.root.clipboard_get()
            if self.is_valid_youtube_url(clipboard_content):
                self.url_var.set(clipboard_content)
                self.log_message(f"✅ 已粘贴URL: {clipboard_content[:50]}...")
            else:
                self.log_message("❌ 剪贴板内容不是有效的YouTube链接")
        except:
            self.log_message("❌ 无法获取剪贴板内容")
            
    def is_valid_youtube_url(self, url):
        youtube_patterns = [
            r'(?:https?://)?(?:www\.)?youtube\.com/watch\?v=[\w-]+',
            r'(?:https?://)?(?:www\.)?youtu\.be/[\w-]+',
            r'(?:https?://)?(?:www\.)?youtube\.com/playlist\?list=[\w-]+',
            r'(?:https?://)?(?:www\.)?youtube\.com/shorts/[\w-]+',
        ]
        return any(re.match(pattern, url) for pattern in youtube_patterns)
        
    def browse_path(self):
        path = filedialog.askdirectory(initialdir=self.output_path_var.get())
        if path:
            self.output_path_var.set(path)
            self.log_message(f"📁 保存路径已更改: {path}")
            
    def open_folder(self):
        path = self.output_path_var.get()
        if os.path.exists(path):
            os.startfile(path)
            self.log_message(f"🔗 已打开文件夹: {path}")
        else:
            messagebox.showwarning("警告", "文件夹不存在")
            
    def add_to_queue(self):
        url = self.url_var.get().strip()
        if not url:
            messagebox.showwarning("警告", "请输入视频链接")
            return
            
        if not self.is_valid_youtube_url(url):
            messagebox.showwarning("警告", "请输入有效的YouTube链接")
            return
            
        for item in self.download_queue:
            if item['url'] == url:
                messagebox.showinfo("提示", "该链接已在队列中")
                return
                
        queue_item = {
            'url': url,
            'quality': self.quality_var.get(),
            'format': self.format_var.get(),
            'status': '⏳ 等待中',
            'downloaded': False
        }
        
        self.download_queue.append(queue_item)
        
        item_id = self.queue_tree.insert('', 'end', values=(
            queue_item['quality'],
            queue_item['format'],
            queue_item['status']
        ))
        
        self.url_var.set("")
        
        self.log_message(f"➕ 已添加到队列: {url[:50]}...")
        
    def clear_queue(self):
        if not self.is_downloading:
            self.download_queue.clear()
            for item in self.queue_tree.get_children():
                self.queue_tree.delete(item)
            self.log_message("🗑️ 队列已清空")
        else:
            messagebox.showwarning("警告", "下载进行中，无法清空队列")
            
    def redownload_selected(self):
        selection = self.queue_tree.selection()
        if not selection:
            messagebox.showwarning("警告", "请选择要重新下载的项目")
            return
            
        if self.is_downloading:
            messagebox.showwarning("警告", "下载进行中，请稍后再试")
            return
            
        selected_item = selection[0]
        item_index = self.queue_tree.index(selected_item)
        
        if item_index < len(self.download_queue):
            self.download_queue[item_index]['status'] = '⏳ 等待中'
            self.download_queue[item_index]['downloaded'] = False
            self.queue_tree.set(selected_item, 'status', '⏳ 等待中')
            
            self.is_downloading = True
            self.download_btn.configure(state='disabled', text='⏸️ 下载中...')
            
            def download_worker():
                try:
                    item = self.download_queue[item_index]
                    self.queue_tree.set(selected_item, 'status', '⬇️ 下载中')
                    
                    success = self.download_single_video(item)
                    
                    status = '✅ 完成' if success else '❌ 失败'
                    self.queue_tree.set(selected_item, 'status', status)
                    item['downloaded'] = success
                    
                finally:
                    self.is_downloading = False
                    self.download_btn.configure(state='normal', text='🚀 开始下载')
                    self.status_var.set("✅ 下载完成")
                    self.progress_var.set(0)
                    
            threading.Thread(target=download_worker, daemon=True).start()
            
    def start_download(self):
        if self.is_downloading:
            messagebox.showwarning("警告", "下载正在进行中")
            return
            
        url = self.url_var.get().strip()
        if url and not self.download_queue:
            messagebox.showinfo("提示", "请先点击'添加到队列'按钮将视频添加到下载列表")
            return
            
        if not self.download_queue:
            messagebox.showwarning("警告", "下载队列为空，请先添加视频")
            return
            
        self.is_downloading = True
        self.download_btn.configure(state='disabled', text='⏸️ 下载中...')
        
        def download_worker():
            try:
                for i, item in enumerate(self.download_queue):
                    if not self.is_downloading:
                        break
                        
                    if item.get('downloaded', False):
                        continue
                        
                    queue_items = self.queue_tree.get_children()
                    if i < len(queue_items):
                        self.queue_tree.set(queue_items[i], 'status', '⬇️ 下载中')
                    
                    success = self.download_single_video(item)
                    
                    if i < len(queue_items):
                        status = '✅ 完成' if success else '❌ 失败'
                        self.queue_tree.set(queue_items[i], 'status', status)
                        item['downloaded'] = success
                        
            finally:
                self.is_downloading = False
                self.download_btn.configure(state='normal', text='🚀 开始下载')
                self.status_var.set("✅ 下载完成")
                self.progress_var.set(0)
                
        threading.Thread(target=download_worker, daemon=True).start()
        
    def download_single_video(self, item):
        url = item['url']
        quality = item['quality']
        format_ext = item['format']
        
        try:
            output_dir = self.output_path_var.get()
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)
                
            self.log_message(f"🚀 开始下载: {url[:50]}...")
            self.status_var.set(f"⬇️ 下载中: {quality} {format_ext}")
            
            if CRYPTO_AVAILABLE:
                downloader = get_encrypted_downloader()
                downloader.start_download()
                
                cmd = downloader.build_download_command(
                    url, quality, format_ext, output_dir,
                    self.concurrent_var.get(), self.download_cover_var.get()
                )
                
                success = downloader.execute_download(
                    cmd, 
                    log_callback=self.log_message,
                    progress_callback=lambda p: self.progress_var.set(p)
                )
                
                if success:
                    self.log_message("✅ 下载成功 [加密核心]")
                    return True
                else:
                    self.log_message("❌ 下载失败 [加密核心]")
                    return False
            else:
                self.log_message("⚠️ 使用备用下载模式")
                return self._fallback_download(url, quality, format_ext, output_dir)
                
        except Exception as e:
            self.log_message(f"❌ 下载出错: {e}")
            return False
    
    def _fallback_download(self, url, quality, format_ext, output_dir):
        try:
            format_selector = self.get_format_selector(quality, format_ext)
            cmd = [
                os.path.join('bin', 'yt-dlp.exe'),
                '--format', format_selector,
                '--output', os.path.join(output_dir, '%(title)s.%(ext)s'),
                '--no-warnings',
                '--newline',
                url
            ]
            
            process = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True,
                encoding='utf-8',
                errors='replace'
            )
            
            while True:
                if not self.is_downloading:
                    process.terminate()
                    return False
                    
                line = process.stdout.readline()
                if not line:
                    break
                    
                line = line.strip()
                if line:
                    self.log_message(line)
                            
            process.wait()
            return process.returncode == 0
                
        except Exception as e:
            self.log_message(f"❌ 备用下载出错: {e}")
            return False
            
    def get_format_selector(self, quality, format_ext):
        if format_ext in ['mp3', 'm4a', 'wav']:
            if format_ext == 'mp3':
                return 'bestaudio[ext=m4a]/bestaudio/best'
            else:
                return f'bestaudio[ext={format_ext}]/bestaudio/best'
        
        if quality == 'best':
            return 'best[ext=mp4]/best[ext=webm]/best'
        elif quality == 'worst':
            return 'worst[ext=mp4]/worst[ext=webm]/worst'
        else:
            height = quality.replace('p', '')
            selectors = [
                f'best[height<={height}][ext=mp4]',
                f'best[height<={height}][ext=webm]',
                f'best[height<={height}]',
                f'bestvideo[height<={height}]+bestaudio/best[height<={height}]',
                'best'
            ]
            
            if format_ext == 'mp4':
                return '/'.join(selectors)
            else:
                format_selectors = [f'{s}[ext={format_ext}]' for s in selectors[:3]]
                format_selectors.extend(selectors)
                return '/'.join(format_selectors)
        
    def stop_download(self):
        if self.is_downloading:
            self.is_downloading = False
            self.log_message("⏹️ 正在停止下载...")
            self.status_var.set("⏹️ 停止中...")
        else:
            messagebox.showinfo("提示", "当前没有进行下载")
            
    def log_message(self, message):
        timestamp = datetime.now().strftime("%H:%M:%S")
        log_entry = f"[{timestamp}] {message}\n"
        
        self.log_text.insert(tk.END, log_entry)
        self.log_text.see(tk.END)
        self.root.update_idletasks()
        
    def run(self):
        if CRYPTO_AVAILABLE:
            author_info = get_author_info()
            self.log_message(f"🎉 {author_info['app_title']}已启动")
            self.log_message(f"👤 作者：{author_info['author_name']} | 网站：{author_info['author_website']}")
        else:
            self.log_message("🎉 YouTube下载器 - 2025版已启动")
            self.log_message("👤 作者信息：[加密保护] | 网站：[加密保护]")
        
        self.log_message("📋 支持格式: MP4, WebM, MKV, MP3, M4A, WAV")
        self.log_message("🎯 支持质量: 144p - 2160p")
        self.log_message("💡 提示: 使用Ctrl+V快速粘贴链接")
        self.log_message("🔒 核心代码已加密保护 - 2025版")
        self.root.mainloop()

def main():
    try:
        app = YouTubeDownloader()
        app.run()
    except Exception as e:
        print(f"程序启动失败: {e}")
        input("按回车键退出...")

if __name__ == "__main__":
    main()